@@ -4,6 +4,7 @@ import (
 	"bytes"
 	"errors"
 	"fmt"
+	"io"
 	"log"
 	"os"
 	"strconv"
@@ -67,6 +68,8 @@ type DockerizedInstanceOptions struct {
 	Image             string
 	RootPassword      string
 	DefaultConnParams string
+	DataBindMount     string // Host path to bind-mount as /var/lib/mysql in container
+	CommandArgs       []string
 }
 
 // CreateInstance attempts to create a Docker container with the supplied name
@@ -113,6 +116,7 @@ func (dc *DockerClient) CreateInstance(opts DockerizedInstanceOptions) (*Dockeri
 		Config: &docker.Config{
 			Image: opts.Image,
 			Env:   env,
+			Cmd:   opts.CommandArgs,
 		},
 		HostConfig: &docker.HostConfig{
 			PortBindings: map[docker.Port][]docker.PortBinding{
@@ -122,6 +126,9 @@ func (dc *DockerClient) CreateInstance(opts DockerizedInstanceOptions) (*Dockeri
 			},
 		},
 	}
+	if opts.DataBindMount != "" {
+		ccopts.HostConfig.Binds = []string{opts.DataBindMount + ":/var/lib/mysql"}
+	}
 	di := &DockerizedInstance{
 		DockerizedInstanceOptions: opts,
 		Manager:                   dc,
@@ -268,7 +275,11 @@ func (di *DockerizedInstance) TryConnect() (err error) {
 	if err != nil {
 		return err
 	}
-	for attempts := 0; attempts < 120; attempts++ {
+	maxAttempts := 120
+	if di.DataBindMount != "" { // bind mounted dir causes slower startup
+		maxAttempts *= 2
+	}
+	for attempts := 0; attempts < maxAttempts; attempts++ {
 		if ok, err = di.Instance.CanConnect(); ok {
 			return err
 		}
@@ -280,7 +291,13 @@ func (di *DockerizedInstance) TryConnect() (err error) {
 // Port returns the actual port number on localhost that maps to the container's
 // internal port 3306.
 func (di *DockerizedInstance) Port() int {
-	portAndProto := docker.Port("3306/tcp")
+	return di.PortMap(3306)
+}
+
+// PortMap returns the port number on localhost that maps to the container's
+// specified internal tcp port.
+func (di *DockerizedInstance) PortMap(containerPort int) int {
+	portAndProto := docker.Port(fmt.Sprintf("%d/tcp", containerPort))
 	portBindings, ok := di.container.NetworkSettings.Ports[portAndProto]
 	if !ok || len(portBindings) == 0 {
 		return 0
@@ -334,6 +351,18 @@ func (di *DockerizedInstance) SourceSQL(filePath string) (string, error) {
 	if di.RootPassword != "" {
 		cmd = append(cmd, fmt.Sprintf("-p%s", di.RootPassword))
 	}
+	stdoutStr, stderrStr, err := di.Exec(cmd, f)
+	stderrStr = strings.Replace(stderrStr, "Warning: Using a password on the command line interface can be insecure.\n", "", 1)
+	if err != nil || strings.Contains(stderrStr, "ERROR") {
+		err = fmt.Errorf("SourceSQL %s: Error sourcing file %s: %s", di, filePath, stderrStr)
+	}
+	return stdoutStr, err
+}
+
+// Exec executes the supplied command/args in the container, blocks until
+// completion, and returns STDOUT and STDERR. An input stream may optionally
+// be supplied for the exec's STDIN.
+func (di *DockerizedInstance) Exec(cmd []string, stdin io.Reader) (string, string, error) {
 	ceopts := docker.CreateExecOptions{
 		AttachStdout: true,
 		AttachStderr: true,
@@ -343,23 +372,22 @@ func (di *DockerizedInstance) SourceSQL(filePath string) (string, error) {
 	}
 	exec, err := di.Manager.client.CreateExec(ceopts)
 	if err != nil {
-		return "", err
+		return "", "", err
 	}
 	var stdout, stderr bytes.Buffer
 	seopts := docker.StartExecOptions{
 		OutputStream: &stdout,
 		ErrorStream:  &stderr,
-		InputStream:  f,
+		InputStream:  stdin,
 	}
 	if err = di.Manager.client.StartExec(exec.ID, seopts); err != nil {
-		return "", err
+		return "", "", err
 	}
-	stdoutStr := stdout.String()
-	stderrStr := strings.Replace(stderr.String(), "Warning: Using a password on the command line interface can be insecure.\n", "", 1)
-	if strings.Contains(stderrStr, "ERROR") {
-		return stdoutStr, fmt.Errorf("SourceSQL %s: Error sourcing file %s: %s", di, filePath, stderrStr)
+	execInfo, err := di.Manager.client.InspectExec(exec.ID)
+	if err == nil && execInfo.ExitCode > 0 {
+		err = fmt.Errorf("Process returned exit code %d", execInfo.ExitCode)
 	}
-	return stdoutStr, nil
+	return stdout.String(), stderr.String(), err
 }
 
 // ContainerNameForImage returns a usable container name (or portion of a name)
