@@ -208,6 +208,10 @@ func flavorTestFiles(flavor Flavor) []string {
 		}
 	}
 
+	if flavor.Min(FlavorMariaDB108) { // descending indexes, IN/OUT/INOUT func params
+		result = append(result, "maria108.sql")
+	}
+
 	return result
 }
 
@@ -234,7 +238,9 @@ func aTable(nextAutoInc uint64) Table {
 func aTableForFlavor(flavor Flavor, nextAutoInc uint64) Table {
 	utf8mb3 := "utf8"
 	utf8mb3DefaultCollation := "utf8_general_ci"
-	if flavor.Min(FlavorMariaDB106) {
+	if flavor.Min(FlavorMySQL80.Dot(29)) {
+		utf8mb3 = "utf8mb3"
+	} else if flavor.Min(FlavorMariaDB106) {
 		utf8mb3 = "utf8mb3"
 		utf8mb3DefaultCollation = "utf8mb3_general_ci"
 	}
